@@ -3,9 +3,12 @@ plot.tri<-function(tri.obj,add=F,xlim=range(tri.obj$x),
 {
   if(!inherits(tri.obj,"tri"))
     stop("tri.obj must be of class \"tri\"")
-  tnabor<-matrix(-1,tri.obj$n,tri.obj$n)
-  storage.mode(tnabor)<-"integer"
-  ans<-.Fortran("troutp",
+  tnabor<- integer(tri.obj$tlnew)
+  nnabs <- integer(tri.obj$n)
+  nptr <- integer(tri.obj$n)
+  nptr1 <- integer(tri.obj$n)
+  nbnos <- integer(tri.obj$n)
+  ans<-.Fortran("troutq",
                  as.integer(tri.obj$nc),
                  as.integer(tri.obj$lc),
                  as.integer(tri.obj$n),
@@ -15,12 +18,15 @@ plot.tri<-function(tri.obj,add=F,xlim=range(tri.obj$x),
                  as.integer(tri.obj$tlptr),
                  as.integer(tri.obj$tlend),
                  as.integer(6),
+                 nnabs=as.integer(nnabs),
+                 nptr=as.integer(nptr),
+                 nptr1=as.integer(nptr1),
                  tnabor=as.integer(tnabor),
+                 nbnos=as.integer(nbnos),
                  na=as.integer(0),
                  nb=as.integer(0),
                  nt=as.integer(0)
                  )
-  ans$tnabor<-matrix(ans$tnabor,tri.obj$n,tri.obj$n)
   if(!add)
     {
       plot.new()
@@ -28,7 +34,7 @@ plot.tri<-function(tri.obj,add=F,xlim=range(tri.obj$x),
     }
   for (i in 1:tri.obj$n)
     {
-      inb<-ans$tnabor[i,ans$tnabor[i,]>0]
+      inb<-ans$tnabor[ans$nptr[i]:ans$nptr1[i]]
       for (j in inb)
         lines(c(tri.obj$x[i],tri.obj$x[j]),c(tri.obj$y[i],tri.obj$y[j]), ...)
     }
