@@ -1,12 +1,15 @@
-summary.tri<-function(tri.obj)
+summary.tri<-function(tri.obj, print=F)
 {
   if(!inherits(tri.obj,"tri"))
     stop("tri.obj must be of class \"tri\"")
-  tnabor<-matrix(-1,tri.obj$n,tri.obj$n)
-  storage.mode(tnabor)<-"integer"
-  ans<-.Fortran("troutp",
+  tnabor<- integer(tri.obj$tlnew)
+  nnabs <- integer(tri.obj$n)
+  nptr <- integer(tri.obj$n)
+  nptr1 <- integer(tri.obj$n)
+  nbnos <- integer(tri.obj$n)
+  ans<-.Fortran("troutq",
                  as.integer(tri.obj$nc),
-                 integer(tri.obj$lc),
+                 as.integer(tri.obj$lc),
                  as.integer(tri.obj$n),
                  as.double(tri.obj$x),
                  as.double(tri.obj$y),
@@ -14,7 +17,11 @@ summary.tri<-function(tri.obj)
                  as.integer(tri.obj$tlptr),
                  as.integer(tri.obj$tlend),
                  as.integer(6),
+                 nnabs=as.integer(nnabs),
+                 nptr=as.integer(nptr),
+                 nptr1=as.integer(nptr1),
                  tnabor=as.integer(tnabor),
+                 nbnos=as.integer(nbnos),
                  na=as.integer(0),
                  nb=as.integer(0),
                  nt=as.integer(0)
