@@ -3,20 +3,31 @@
     
     dummy.node<-function(x0,y0,x1,y1,x2,y2,d)
       {
-        #determine a direction orthogonal to p1--p2
-        # two versions
+        # determine a direction orthogonal to p1--p2
         #
-        dy<-  x2-x1
-        dx<- -y2+y1
+        #              p_1
+        #               |
+        #               |d
+        #    p_0 ------>+ - - - - -> dummy_node
+        #          r    |
+        #               V
+        #              p_2------->
+        #                    n
+        # two versions, r and n
+        #
+
+        dx<-  x2-x1
+        dy<-  y2-y1
+        nx<- -dy
+        ny<-  dx
         
         rx<-(x1+x2)/2-x0
         ry<-(y1+y2)/2-y0
         
         lr<-sqrt(rx^2+ry^2)
-        ld<-sqrt(dx^2+dy^2)
-        # use the numerically better version
-        # if(lr > ld)
-        if(F)
+        ln<-sqrt(nx^2+ny^2)
+        # choose the numerically better version
+        if(lr > ln)
           {
             vx<-rx/lr
             vy<-ry/lr
@@ -28,8 +39,8 @@
           }
         else
           {
-            vx<-dx/ld
-            vy<-dy/ld
+            vx<-nx/ln
+            vy<-ny/ln
             eps<-1e-7
             if(in.convex.hull(ret$tri,(x1+x2)/2+eps*vx,(y1+y2)/2+eps*vy))
               d <- - d
@@ -42,8 +53,10 @@
     tri.obj<-tri.mesh(x=x,y=y,duplicate=duplicate)
     nt<-summary(tri.obj)$nt
     tmptri<-matrix(0,9,2*nt)
-    lccc<-matrix(0,9,nt)
+    lccc<-matrix(0,4,nt)
     storage.mode(lccc)<-"double"
+    iccc<-matrix(0,6,nt)
+    storage.mode(iccc)<-"integer"
     ans<-.Fortran("voronoi",
                   as.integer(tri.obj$nc),
                   as.integer(tri.obj$lc),
@@ -55,13 +68,23 @@
                   as.integer(tri.obj$tlend),
                   as.integer(nt),
                   lccc=as.double(lccc),
+                  iccc=as.integer(iccc),
                   lct=integer(tri.obj$nc),
                   as.integer(tmptri),
                   ier=as.integer(0))
-    tmp<-matrix(ans$lccc,nt,9,byrow=T)
-    ret<-list(x=tmp[,1],y=tmp[,2],node=(tmp[,3]>0),area=tmp[,3],
-              n1=tmp[,4],n2=tmp[,5],n3=tmp[,6],
-              p1=tmp[,7],p2=tmp[,8],p3=tmp[,9],
+    lccc<-matrix(ans$lccc,nt,4,byrow=T)
+    iccc<-matrix(ans$iccc,nt,6,byrow=T)
+    ret<-list(x=lccc[,1],
+              y=lccc[,2],
+              node=(lccc[,3]>0),
+              area=lccc[,3],
+              ratio=lccc[,4],
+              n1=iccc[,1],
+              n2=iccc[,2],
+              n3=iccc[,3],
+              p1=iccc[,4],
+              p2=iccc[,5],
+              p3=iccc[,6],
               tri=tri.obj)
 
     ret$dummy.x<-integer(0)
@@ -69,7 +92,7 @@
     dummy.cnt<-0
     dmax<-max(diff(range(ret$x)),diff(range(ret$x)))
     n<-length(ret$x)
-    # add dummy node on the border of the triangulation
+    # add dummy nodes on the border of the triangulation
     for (i in 1:n)
       {
         if(ret$node[i])
@@ -119,10 +142,37 @@
                   }
                 else
                   {
-                    #  Two edges of i coincide with border.
-                    #  (= 3 points on hull )
-                    #  what to do here ?
-                    tmp<-0
+                    # Two edges of i coincide with border.
+                    # (= 3 points on hull )
+                    # that means this triangle forms one corner of
+                    # the convex hull
+                    # Find out which edge of triangle i is not
+                    # on the border: (check if midpoints of edges lay
+                    # on hull)
+                    tr<-c(ret$p1[i],ret$p2[i],ret$p3[i])
+                    edge<-list(from=tr[c(1,2,3)],to=tr[c(2,3,1)])
+                    mx <- ret$tri$x[edge$from]-ret$tri$x[edge$to]/2
+                    my <- ret$tri$y[edge$from]-ret$tri$y[edge$to]/2
+                    eonb <- on.convex.hull(ret$tri,mx,my)
+                    # Find two dummy nodes
+                    for (id in 1:3){
+                      if (eonb[id]){
+                        pn<-dummy.node(ret$x[i],ret$y[i],
+                                       ret$tri$x[edge$from[id]],
+                                       ret$tri$y[edge$from[id]],
+                                       ret$tri$x[edge$to[id]],
+                                       ret$tri$y[edge$to[id]],
+                                       dmax)
+                        dummy.cnt<- dummy.cnt+1
+                        ret$dummy.x[dummy.cnt]<-pn$x
+                        ret$dummy.y[dummy.cnt]<-pn$y
+                        # update neighbour relation
+                        # (negative index indicates dummy node)
+                        if(ret$n1[i]==0) ret$n1[i]<- -dummy.cnt
+                        if(ret$n2[i]==0) ret$n2[i]<- -dummy.cnt
+                        if(ret$n3[i]==0) ret$n3[i]<- -dummy.cnt
+                      }
+                    }
                   }
               }
           }
